// M30xFun.h: interface for the CM30xFun class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_M30XFUN_H__791C4DD1_2961_43C6_9704_D538BA0FBF43__INCLUDED_)
#define AFX_M30XFUN_H__791C4DD1_2961_43C6_9704_D538BA0FBF43__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "M30xDemo.h"

#define RETRY_MAX					0x5
#define RESET_RETRY_MAX				0x03

#define DO_READ						0x01
#define DO_WRITE					0x00
#define CMND_SIGN_ON				0x00
#define CMND_RESET					0x01
#define CMND_DIG_OUT				0x02
#define CMND_CHAN_CHG				0x04
#define CMND_ANALOG_OUT				0x0a
#define CMND_GET_CONVERSION			0x81
#define CMND_SET_MODE				0x84
#define CMND_SYS_CAL				0x85
#define CMND_CHECKSUM_SCAN			0x8D
#define NO_ARG						0x00

#define	SINGLE_CHAN_SCAN			0x01
#define	MULTI_CHAN_SCAN				0x02
#define	MULTI_CHAN_CAL_SCAN			0x03
#define	SINGLE_CHAN_DIGIN_SCAN		0x04
#define	MULTI_CHAN_DIGIN_SCAN		0x05
#define	MULTI_CHAN_CAL_DIGIN_SCAN	0x06

#define SCAN_STATUS_UKNOWN				0x00	// Driver status unknown
#define SCAN_RUNNING 					0x01	// Driver scan thread is running
#define SCAN_DATA_REQUEST_SUCCEEDED		0x02	// Pending driver to device data request succeeded
#define SCAN_ENDING 					0x04	// DLL has requested driver to end scanning
#define SCAN_HALTED 					0x08	// Driver scan thread was halted (possible I/O err, device unplug, etc)
#define SCAN_CHECKSUM_ERROR				0x10	// Driver's data checksum doesn't agree with device's checksum
#define SCAN_VOLTAGE_BUFFER_WRAP		0x20	// The circular data buffer shared between DLL and Driver has wrapped
#define SCAN_MICRO_CODE_BUFF_WRAP		0x40	// The device's (hardware) internal buffer has wrapped
#define SCAN_DEVICE_IO_ERROR			0x80	// Driver received an I/O error
#define SCAN_DEVICE_REMOVED				0x100	// The USB cable has been disconnected
#define SCAN_DEVICE_NOT_FOUND			0x200	// The DLL was unable to find the requested device to begin the scan

#define ARG_IGNORED						0x00

#define MAX_NUM_DEVICE					32
#define VOLTAGE_ARRAY_SIZE				4096

typedef enum {
	COMMAND_SUCCESS = 0,
	CALIBERATE_DEV_FAILED,
	CHECK_DLL_LIBRARY_FAILED,
	DEV_TIME_OUT,
	DEV_WAS_REMOVED,
	DEV_INVALID_ID_ENTRY,
	DEV_NOT_IN_LIST,
	DLL_FAILED,
	FIND_FUNCTION_IN_DLL_FAILED,
	GET_ECHO_FAILED,
	INIT_DEV_FAILED,
	INVALID_DEV_ID,
	INVALID_DEV_RESPONSE,
	INVALID_ARGUMENT,
	INIT_DEV_SUCCESS,
	LOAD_DLL_LIBRARY_FAILED,
	MEMORY_ALLOC_FAILED,
	RESET_DEV_FAILED,
	READ_CONFIG_FILE_FAILED,
	REINIT_DEV_FAILED,
	READ_CONVERSION_FAILED,
	REV_DATA_FAILED,
	SCAN_HALTED_ERR,
	SEND_CMD_FAILED,
	SEND_DATA_FAILED,
	SIGNON_DEV_FAILED,
	SIGN_ON_SUCCESS,
	SET_RATE_FAILED,
	SET_DLL_APP_MSG_FAILED,
	VERIFY_CHKSUM_FAILED,
	VERIFY_DEV_FAILED,
	DEV_NO_POWER
}RETURN_CODE;

typedef struct _SCAN_OBJECT_DIO {
	BYTE			SO_bScanType;			// Definitions below
	double			SO_dblRate;			// device rate			    	
	UINT			SO_uiTotalPointsReadByDrvr;	// points read by driver.	
	UINT			SO_uiPointsInBuffer;		// points in DLL/Driver shared buffer.
	UINT			SO_uiStatus;			// Definitions below
	UINT			SO_uiSizeVoltageArray;		// The size of app's voltage array. 
	UINT			SO_uiCurHead;			// The device driver increments whenever it adds a byte to DLL/Driver buffer
	UINT			SO_uiCurTail;			// The DLL increments this whenever it reads a byte DLL/Driver buffer
   	BYTE			SO_bDigitalInput;		// Current digital input value. Driver updates this every 10th point read
	BYTE			SO_bChecksum;		// Driver's current checksum
   	BYTE			SO_bDeviceDisconnect; 	// required but value is ignored (backward compatibility)
	UINT			SO_clkScanStartTime;		// not currently changed
	UINT			SO_clkScanEndTime;		// not currently changed
	BYTE			SO_bScanArg;			// Number of channels to scan.
} SCAN_OBJECT_DIO, *PSCAN_OBJECT_DIO;

class CM30xFun  
{
public:
	RETURN_CODE SendReset(USHORT usDevID);
	CM30xFun();
	virtual ~CM30xFun();
	RETURN_CODE CheckPower(USHORT usDevID);
	RETURN_CODE GetConversion(USHORT usDevID,double* volts_result);
	RETURN_CODE EndScan(USHORT usDevID);
	RETURN_CODE SysCal(USHORT usDevID);
	RETURN_CODE AnalogOutPut(USHORT usDevID, double voltage,BYTE bDacChan);
	RETURN_CODE DigiOutPut(USHORT usDevID,BYTE bDigOutVal);
	RETURN_CODE ChangeChan(USHORT usDevID,BYTE bCurChan);
	RETURN_CODE SendRate(USHORT usDevID,double* fRate);
	RETURN_CODE SendSignon(USHORT usDevID);
	RETURN_CODE CheckSignon(USHORT usDevID);
	RETURN_CODE ReadScanCount(USHORT usDevID,UINT *ChanCount);
	RETURN_CODE StartScan(USHORT usDevID,_SCAN_OBJECT_DIO* p_ScanObj);
	RETURN_CODE SendDigitalOut(USHORT usDevID,BYTE bDigOut);
	RETURN_CODE SendDACOut(USHORT usDevID,double fVolts, BYTE bChanIndex);
	BOOL CalcRateCodes(BYTE* bLow, BYTE* bMid, BYTE* bHigh, double* fRate);
	BYTE GetScanDlgID(USHORT usDevID);

	UINT *p_ulRawCntsBuff;
	UINT ulRawCntsBuff[VOLTAGE_ARRAY_SIZE];
	BYTE bDiginBuff[VOLTAGE_ARRAY_SIZE];
	BYTE* p_bDiginBuff;
	BYTE bLastDigin;
	UINT NumBytesToReadWrite;
	BYTE bHostClientDataBuf_WRITE[8];
	BYTE bHostClientDataBuf_READ[32];
	BYTE* p_bHostClientDataBuf_WRITE;
	BYTE* p_bHostClientDataBuf_READ;
private:
	CString TmpStr;
	BYTE bLow, bMid, bHigh;
};

#endif // !defined(AFX_M30XFUN_H__791C4DD1_2961_43C6_9704_D538BA0FBF43__INCLUDED_)
